@@ -1,25 +1,28 @@
-/* keyring.c  -  The GNU Privacy Assistant
- *      Copyright (C) 2000, 2001 G-N-U GmbH.
- *
- * This file is part of GPA
- *
- * GPA is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * GPA is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
- */
+/* keyring.c - The GNU Privacy Assistant keyring.
+   Copyright (C) 2000, 2001 G-N-U GmbH.
+   Copyright (C) 2005 g10 Code GmbH.
+
+   This file is part of GPA
+
+   GPA is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
+
+   GPA is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with GPA; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA  */
+
+#if HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include "gpa.h"
-#include <config.h>
 #include <gpgme.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
@@ -293,6 +296,15 @@ gpa_keyring_editor_changed_wot_cb (gpointer data)
   gpa_keylist_start_reload (editor->keylist);  
 }
 
+static void
+gpa_keyring_editor_changed_wot_secret_cb (gpointer data)
+{
+  GPAKeyringEditor *editor = data;
+
+  gpa_keylist_imported_secret_key (editor->keylist);
+  gpa_keylist_start_reload (editor->keylist);  
+}
+
 static void
 gpa_keyring_editor_key_modified (GpaKeyEditDialog *dialog, gpgme_key_t key,
 				 gpointer data)
@@ -301,6 +313,7 @@ gpa_keyring_editor_key_modified (GpaKeyEditDialog *dialog, gpgme_key_t key,
   gpa_keylist_start_reload (editor->keylist);  
 }
 
+
 static void
 gpa_keyring_editor_new_key_cb (gpointer data, const gchar *fpr)
 {
@@ -327,10 +340,15 @@ register_import_operation (GPAKeyringEditor * editor, GpaImportOperation *op)
   g_signal_connect_swapped (G_OBJECT (op), "imported_keys",
 			    G_CALLBACK (gpa_keyring_editor_changed_wot_cb),
 			    editor);
+  g_signal_connect_swapped 
+    (G_OBJECT (op), "imported_secret_keys",
+     G_CALLBACK (gpa_keyring_editor_changed_wot_secret_cb),
+     editor);
   g_signal_connect (G_OBJECT (op), "completed",
 		    G_CALLBACK (g_object_unref), editor); 
 }
 
+
 static void
 register_generate_operation (GPAKeyringEditor * editor, GpaGenKeyOperation *op)
 {
@@ -397,20 +415,20 @@ key_has_been_signed (const gpgme_key_t key,
 static gboolean
 keyring_editor_can_sign (gpointer param)
 {
-  GPAKeyringEditor * editor = param;
   const gpgme_key_t default_key = gpa_options_get_default_key
     (gpa_options_get_instance ());
   gboolean result = FALSE;
 
-  if (keyring_editor_has_single_selection (param) && default_key)
+  if (default_key && keyring_editor_has_single_selection (param))
     {
       /* the most important requirements have been met, now find out
        * whether the selected key was already signed with the default
        * key */
+      GPAKeyringEditor * editor = param;
       gpgme_key_t key = keyring_editor_current_key (editor);
       result = !key_has_been_signed (key, default_key);
     }
-  else if (keyring_editor_has_selection (param))
+  else if (default_key && keyring_editor_has_selection (param))
     {
       /* Always allow signing many keys at once.
        */
@@ -806,6 +824,15 @@ keyring_editor_copy (gpointer param)
     }
 }
 
+/* Reload the key list */
+static void
+keyring_editor_refresh (gpointer param)
+{
+  GPAKeyringEditor * editor = param;
+
+  gpa_keylist_start_reload (editor->keylist);
+}
+
 /* Create and return the menu bar for the key ring editor */
 static GtkWidget *
 keyring_editor_menubar_new (GtkWidget * window,
@@ -833,6 +860,9 @@ keyring_editor_menubar_new (GtkWidget * window,
   };
   GtkItemFactoryEntry keys_menu[] = {
     {_("/_Keys"), NULL, NULL, 0, "<Branch>"},
+    {_("/Keys/_Refresh"), NULL, keyring_editor_refresh, 0,
+     "<StockItem>", GTK_STOCK_REFRESH},
+    {_("/Keys/sep0"), NULL, NULL, 0, "<Separator>"},
     {_("/Keys/_New Key..."), NULL, keyring_editor_generate_key, 0,
      "<StockItem>", GTK_STOCK_NEW},
     {_("/Keys/_Delete Keys..."), NULL, keyring_editor_delete, 0,
@@ -1199,14 +1229,13 @@ keyring_details_notebook (GPAKeyringEditor *editor)
   return notebook;
 }
 
-/* Fill the details page of the details notebook with the properties of
- * the publix key key */
+/* Fill the details page of the details notebook with the properties
+   of the publix key key */
 static void
 keyring_details_page_fill_key (GPAKeyringEditor * editor, gpgme_key_t key)
 {
-  gchar * text;
-  gchar * uid;
-  gint i;
+  gpgme_user_id_t uid;
+  gchar *text;
 
   if (gpa_keytable_lookup_key (gpa_keytable_get_secret_instance(), 
 			       key->subkeys->fpr) != NULL)
@@ -1222,21 +1251,26 @@ keyring_details_page_fill_key (GPAKeyringEditor * editor, gpgme_key_t key)
   gtk_label_set_text (GTK_LABEL (editor->detail_capabilities),
 		      gpa_get_key_capabilities_text (key));
 
-  /* One user ID on each line */
-  text = gpa_gpgme_key_get_userid (key, 0);
-  for (i = 1; (uid = gpa_gpgme_key_get_userid (key, i)) != NULL; i++)
+  /* One user ID on each line.  */
+  text = gpa_gpgme_key_get_userid (key->uids);
+  if (key->uids)
     {
-      gchar *tmp = text;
-      text = g_strconcat (text, "\n", uid, NULL);
-      g_free (tmp);
+      for (uid = key->uids->next; uid; uid = uid->next)
+        {
+          gchar *uid_string = gpa_gpgme_key_get_userid (uid);
+          gchar *tmp = text;
+          text = g_strconcat (text, "\n", uid_string, NULL);
+          g_free (tmp);
+          g_free (uid_string);
+        }
     }
   gtk_label_set_text (GTK_LABEL (editor->detail_name), text);
   g_free (text);
 
-  text = (gchar*) gpa_gpgme_key_get_short_keyid (key, 0);
+  text = (gchar*) gpa_gpgme_key_get_short_keyid (key);
   gtk_label_set_text (GTK_LABEL (editor->detail_key_id), text);
 
-  text = gpa_gpgme_key_get_fingerprint (key, 0);
+  text = gpa_gpgme_key_format_fingerprint (key->subkeys->fpr);
   gtk_label_set_text (GTK_LABEL (editor->detail_fingerprint), text);
   g_free (text);
   text = gpa_expiry_date_string (key->subkeys->expires);
@@ -1276,7 +1310,10 @@ keyring_details_page_fill_num_keys (GPAKeyringEditor * editor, gint num_key)
     }
   else
     {
-      gchar * text = g_strdup_printf (_("%d keys selected"), num_key);
+      char *text = g_strdup_printf (ngettext("%d key selected",
+                                             "%d keys selected",
+                                             num_key), num_key); 
+
       gtk_label_set_text (GTK_LABEL (editor->details_num_label), text);
       g_free (text);
     }
@@ -1297,22 +1334,28 @@ keyring_signatures_page_fill_key (GPAKeyringEditor * editor, gpgme_key_t key)
 {
   GtkWidget *menu;
   GtkWidget *label;
-  gchar *uid;
-  int i;
 
-  /* Create the menu for the popdown UID list, if there is more than une UID
-   */
+  /* Create the menu for the popdown UID list, if there is more than
+     one UID.  */
   if (key->uids && key->uids->next)
     {
+      gpgme_user_id_t uid;
+
       menu = gtk_menu_new ();
       label = gtk_menu_item_new_with_label (_("All signatures"));
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), label);
-      for (i = 0; (uid = gpa_gpgme_key_get_userid (key, i)) != NULL; i++)
+
+      uid = key->uids;
+      while (uid)
 	{
-	  label = gtk_menu_item_new_with_label (uid);
+	  gchar *uid_string = gpa_gpgme_key_get_userid (uid);
+	  label = gtk_menu_item_new_with_label (uid_string);
 	  gtk_menu_shell_append (GTK_MENU_SHELL (menu), label);
-	  g_free (uid);
+	  g_free (uid_string);
+
+	  uid = uid->next;
 	}
+
       gtk_option_menu_set_menu (GTK_OPTION_MENU (editor->signatures_uids), 
 				menu);
       gtk_widget_show_all (menu);
@@ -1482,6 +1525,12 @@ toolbar_preferences (GtkWidget *widget, gpointer param)
 }
 #endif
 
+static void
+toolbar_refresh (GtkWidget *widget, gpointer param)
+{
+  keyring_editor_refresh (param);
+}
+
 static GtkWidget *
 keyring_toolbar_new (GtkWidget * window, GPAKeyringEditor *editor)
 {
@@ -1588,6 +1637,14 @@ keyring_toolbar_new (GtkWidget * window, GPAKeyringEditor *editor)
 
   gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
 
+  item = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_REFRESH,
+                                   _("Refresh the keyring"),
+                                   _("refresh keyring"),
+                                   GTK_SIGNAL_FUNC (toolbar_refresh),
+                                   editor, -1);
+
+  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
+
   icon = gpa_create_icon_widget (window, "openfile");
   item = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), _("Files"),
                                   _("Open the File Manager"),
@@ -1634,15 +1691,15 @@ static void
 keyring_update_status_bar (GPAKeyringEditor * editor)
 {
   gpgme_key_t key = gpa_options_get_default_key (gpa_options_get_instance ());
-  gchar *string;
 
   if (key)
     {
-      string =  gpa_gpgme_key_get_userid (key, 0);
+      gchar *string = gpa_gpgme_key_get_userid (key->uids);
+
       gtk_label_set_text (GTK_LABEL (editor->status_key_user), string);
       g_free (string);
       gtk_label_set_text (GTK_LABEL (editor->status_key_id),
-                          gpa_gpgme_key_get_short_keyid (key, 0));
+                          gpa_gpgme_key_get_short_keyid (key));
     }
   else
     {
